/*
 *
 * Copyright (C) 2000  Robert Olsson, Jens Ls 
 * Swedish University of Agricultural Sciences
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

/* 
 *======== Debugging macros =============
 */

#ifndef ZBUG

#define ZBUG_MAIN unsigned int zdebug = 0; \
unsigned int zdebug_s = 0; \
char **zdebug_array = 0;\
FILE *zbug_f = 0;\
unsigned int zbug_g = 0;

#define ZBUG_EXTERN extern unsigned int zdebug;\
extern unsigned int zdebug_s; \
extern char **zdebug_array;\
extern FILE *zbug_f;\
extern unsigned int zbug_g;

/*
 * Local debugging types
 * Maximum number of types are sizeof( zdebug ) * 8
 */

enum
{
   NO_ZDEBUG, /* Dont remove */
   ZBUG_PIM_AUTORP,
   ZBUG_PIM_PACKET,
   ZBUG_PIM_PACKET_V1,
   ZBUG_PIM_PACKET_V2,
   ZBUG_PIM_NEIGHBOR,
   ZBUG_PIM_ROUTE,
   ZBUG_PIM_REGISTER_PACKET,
   ZBUG_PIM_KERNEL_PACKET,
   ZBUG_PIM_MESSAGES,
   ZBUG_PIM_TIMER,
   ZBUG_IGMP_MESSAGES,
   ZBUG_IGMP_TIMER,
   MAX_ZBUG /* Dont remove */
};

/*
 * This defines the function through which output is sent
 */
#define ZBUGF printf

/*
 * zbug logfile for group specific debugging. ( ZBUG_G() )
 */
#define ZBUG_LOGFILE "/tmp/zbug.log"

/*
 * ------ Macros for usage
 */

/* Open log */
#define ZBUG_OPEN if(!zbug_f) zbug_f = fopen(ZBUG_LOGFILE, "w")
#define ZBUG_CLOSE if(zbug_f) { fclose(zbug_f); zbug_f = 0; }
#define ZBUG_LOG zbug_f

#define ZBUG_LOG_GROUP( GROUP ) zbug_g = GROUP; \
                               if(GROUP) { if(!zbug_f) zbug_f = fopen(ZBUG_LOGFILE, "w");}\
                                         else if(zbug_f) { fclose(zbug_f); zbug_f = 0; }

/* Initialize array */
#define ZBUG_INIT() zdebug_array = (char **) malloc( MAX_ZBUG * sizeof( char *) ); \
                    memset( zdebug_array, 0, MAX_ZBUG * sizeof( char *))
/* Define name of debuginfo in array */
#define ZBUG_DEF( TYPE ) zdebug_array[ TYPE ] = #TYPE
/*
 * Check which infotype the given name is associated with
 * Case insensitive
 */
#define ZBUG_TYPEOF( TYPE, NAME ) { int __i; for(__i = 1; __i<MAX_ZBUG; __i++) \
                  if(zdebug_array[__i]) if(!strcasecmp( zdebug_array[__i], NAME )) break;\
                  if(__i == MAX_ZBUG) TYPE = 0; else TYPE = __i; }

#define ZBUG_ON_RANGE( TYPE ) { int __i, __l, __m=0; \
                  __l = strlen(zdebug_array[TYPE]); \
                  for(__i = 1; __i<MAX_ZBUG; __i++) \
                  if(zdebug_array[__i]) \
                        if(!strncasecmp( zdebug_array[__i], zdebug_array[TYPE], __l )) \
                               __m |= (1 <<(__i-1)); \
                  zdebug |= __m; zdebug_s |= 1 << (TYPE-1); }
#define ZBUG_OFF_RANGE( TYPE ) { int __i, __l, __m=0; __l = strlen(zdebug_array[TYPE]); \
                               for(__i = 1; __i<MAX_ZBUG; __i++) \
                                if(zdebug_array[__i]) \
                         if(!strncasecmp( zdebug_array[__i], zdebug_array[TYPE], __l )) __m |= (1 <<(__i-1)); \
                                zdebug_s &= ~(1 << (TYPE-1)); zdebug &= ~__m; zdebug |= (zdebug_s & __m); }

/* Debug info output macros */
#define ZBUG_ON( TYPE ) zdebug |= (1 << (TYPE-1)); zdebug_s |= (1<<(TYPE-1)) /* Turn ON debugging for TYPE */
#define ZBUG_OFF( TYPE ) zdebug &= ~( 1 << (TYPE-1)); zdebug_s &= ~(1<<(TYPE-1)) /* Turn OFF debugging for TYPE */

#define ZBUG(TYPE) if(zdebug & (1 << (TYPE-1))) 
#define ZBUG_PRINT(TYPE, ARGS...) \
      if(zdebug) if (zdebug & (1 << (TYPE-1))) \
        { ZBUGF ( "%s> ", #TYPE); \
        ZBUGF ( ##ARGS ); }

#define ZBUG_G(GROUP) if((zbug_g == GROUP) && zbug_f)
#endif

/*
 * Examples:
 * 
 *    ZBUG_ON( THIS );
 *    ZBUG_PRINT( THIS, "Status is %d!!\n", i);
 *    ZBUG_PRINT( THIS, "Recieved %d::%s!!\n", i, "jens");
 *    ZBUG( THIS )
 *      ZBUGF( "Report %d\n", i);
 *
 * With definitions of names:
 * 
 *    ZBUG_INIT();
 *    ZBUG_DEF( THIS );
 *    ZBUG_TYPEOF( type, "this" );
 *    if(type) printf("Type matched was: %d", type);
 */









